// interface definitions
// ---------------------
import * as dntShim from "./_dnt.shims.js";
var UrlCollectionItemType;
(function (UrlCollectionItemType) {
    UrlCollectionItemType["String"] = "STRING";
    UrlCollectionItemType["RegExp"] = "REGEXP";
    UrlCollectionItemType["URLPattern"] = "URLPATTERN";
})(UrlCollectionItemType || (UrlCollectionItemType = {}));
class UrlCollection {
    constructor() {
        Object.defineProperty(this, "items", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.items = [];
    }
    add(methods, requestUrlPattern, data) {
        const methods_ = methods.constructor === Array
            ? methods
            : [methods];
        const requestUrlPatternType = requestUrlPattern.constructor === RegExp
            ? UrlCollectionItemType.RegExp
            : requestUrlPattern.constructor === dntShim.URLPattern
                ? UrlCollectionItemType.URLPattern
                : UrlCollectionItemType.String;
        this.items.push({
            methods: methods_,
            requestUrlPattern,
            requestUrlPatternType,
            data,
        });
    }
    clear() {
        this.items = [];
    }
    filter(method, uri, urlConverter) {
        const foundItems = this.items
            .filter((x) => x.methods.includes(method))
            .filter((x) => {
            const uri_ = typeof uri === "function" ? uri() : uri;
            if (x.requestUrlPatternType === UrlCollectionItemType.RegExp) {
                return x.requestUrlPattern.test(uri_);
            }
            else if (x.requestUrlPatternType === UrlCollectionItemType.URLPattern) {
                return x.requestUrlPattern.test(uri_);
            }
            if (urlConverter !== undefined) {
                const convertedUrl = urlConverter(x.requestUrlPattern).toString();
                return convertedUrl === uri;
            }
            return x.requestUrlPattern === uri;
        });
        return foundItems;
    }
    filterByRequest(request, urlConverter) {
        return this.filter(request.method, request.url, urlConverter);
    }
}
export { UrlCollection, };
