"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MockRegistry = exports.HookRegistry = exports.FetchpStatus = exports.FetchpHookType = exports.default = exports.fetchp = exports.Fetchp = exports.CacheRegistry = void 0;
const hook_registry_js_1 = require("./hook-registry.js");
Object.defineProperty(exports, "FetchpHookType", { enumerable: true, get: function () { return hook_registry_js_1.FetchpHookType; } });
Object.defineProperty(exports, "HookRegistry", { enumerable: true, get: function () { return hook_registry_js_1.HookRegistry; } });
const mock_registry_js_1 = require("./mock-registry.js");
Object.defineProperty(exports, "MockRegistry", { enumerable: true, get: function () { return mock_registry_js_1.MockRegistry; } });
const cache_registry_js_1 = require("./cache-registry.js");
Object.defineProperty(exports, "CacheRegistry", { enumerable: true, get: function () { return cache_registry_js_1.CacheRegistry; } });
var FetchpStatus;
(function (FetchpStatus) {
    FetchpStatus["IDLE"] = "idle";
    FetchpStatus["PREPARING"] = "preparing";
    FetchpStatus["FETCHING"] = "fetching";
    FetchpStatus["LOADING"] = "loading";
    FetchpStatus["ERROR"] = "error";
    FetchpStatus["CANCELED"] = "canceled";
    FetchpStatus["SUCCESS"] = "success";
})(FetchpStatus || (FetchpStatus = {}));
exports.FetchpStatus = FetchpStatus;
class Fetchp {
    constructor() {
        Object.defineProperty(this, "baseUri", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "hooks", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "mocks", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "cache", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.baseUri = undefined;
        this.hooks = new hook_registry_js_1.HookRegistry();
        this.mocks = new mock_registry_js_1.MockRegistry();
        this.cache = new cache_registry_js_1.CacheRegistry();
    }
    setBaseUri(uri) {
        this.baseUri = uri;
    }
    internalMockChecker(request) {
        var _a;
        const mock = this.mocks.find(request, (uri) => this.internalUrlConverter(uri));
        return (_a = mock === null || mock === void 0 ? void 0 : mock.responseFn) === null || _a === void 0 ? void 0 : _a.call(mock, request);
    }
    internalCacheChecker(request) {
        var _a;
        const cache = this.cache.items.filterByRequest(request, (uri) => this.internalUrlConverter(uri));
        return (_a = cache === null || cache === void 0 ? void 0 : cache[0]) === null || _a === void 0 ? void 0 : _a.data;
    }
    internalFetcher(request) {
        return fetch(request);
    }
    internalDataDeserializer(response, cacheMode) {
        var _a;
        const contentType = (_a = response.headers) === null || _a === void 0 ? void 0 : _a.get("content-type");
        const _res = cacheMode ? response.clone() : response;
        if (contentType !== null &&
            contentType !== undefined &&
            contentType.startsWith("application/json")) {
            return _res.json();
        }
        return _res.text();
    }
    internalUrlConverter(uri, params) {
        const finalUri = typeof uri === "function" ? uri(params) : uri;
        return new URL(finalUri, this.baseUri);
    }
    internalAwaiterGenerator() {
        let resolveFnc;
        const promise = new Promise((resolve) => {
            resolveFnc = resolve;
        });
        return [resolveFnc, promise];
    }
    internalRequestStep1InitHeaders(headers, init, callback) {
        var _a;
        return Promise.all([
            FetchpStatus.PREPARING,
            undefined,
            undefined,
            callback(FetchpStatus.PREPARING),
            (_a = init === null || init === void 0 ? void 0 : init.statusCallback) === null || _a === void 0 ? void 0 : _a.call(init, FetchpStatus.PREPARING),
            this.hooks.callGlobalHooks(hook_registry_js_1.FetchpHookType.StateChange, undefined, FetchpStatus.PREPARING),
            this.hooks.callGlobalHooks(hook_registry_js_1.FetchpHookType.BuildRequestHeaders, headers),
        ]);
    }
    internalRequestStep2InitRequest(method, url, headers, init, abortController, callback) {
        const requestInit = Object.assign(Object.assign({ method, signal: abortController.signal }, (init !== null && init !== void 0 ? init : {})), { headers });
        const request = new Request(url, requestInit);
        return Promise.all([
            FetchpStatus.PREPARING,
            request,
            undefined,
            undefined,
            callback(FetchpStatus.PREPARING, request),
            this.hooks.callHooksWithRequest(hook_registry_js_1.FetchpHookType.NewRequest, request, this.internalUrlConverter, request),
        ]);
    }
    internalRequestStep3CheckForMocks(request, abortController, _callback) {
        const mockedResponse = request !== undefined && !abortController.signal.aborted
            ? this.internalMockChecker(request)
            : undefined;
        return Promise.all([
            FetchpStatus.PREPARING,
            request,
            mockedResponse,
            undefined,
            // callback(FetchpStatus.PREPARING),
        ]);
    }
    internalRequestStep4CheckForCache(request, mockedResponse, abortController, _callback) {
        const mockedOrCachedResponse = request === undefined ||
            abortController.signal.aborted ||
            mockedResponse !== undefined
            ? mockedResponse
            : this.internalCacheChecker(request);
        return Promise.all([
            FetchpStatus.PREPARING,
            request,
            mockedOrCachedResponse,
            undefined,
            // callback(FetchpStatus.PREPARING),
        ]);
    }
    internalRequestStep5ExecuteRequest(request, mockedOrCachedResponse, init, abortController, callback) {
        var _a, _b;
        if (request !== undefined &&
            mockedOrCachedResponse === undefined &&
            !abortController.signal.aborted) {
            const fetchedResponse = this.internalFetcher(request);
            if ((init === null || init === void 0 ? void 0 : init.cacheRequest) === true) {
                this.cache.items.add((_a = request.method) !== null && _a !== void 0 ? _a : "GET", request.url, fetchedResponse);
            }
            return Promise.all([
                FetchpStatus.FETCHING,
                request,
                fetchedResponse,
                undefined,
                callback(FetchpStatus.FETCHING),
                (_b = init === null || init === void 0 ? void 0 : init.statusCallback) === null || _b === void 0 ? void 0 : _b.call(init, FetchpStatus.FETCHING),
                this.hooks.callHooksWithRequest(hook_registry_js_1.FetchpHookType.StateChange, request, this.internalUrlConverter, request, FetchpStatus.FETCHING),
            ]);
        }
        return Promise.all([
            FetchpStatus.PREPARING,
            request,
            mockedOrCachedResponse,
            undefined,
            // callback(FetchpStatus.PREPARING),
        ]);
    }
    internalRequestOnError(request, 
    // deno-lint-ignore no-explicit-any
    error, 
    // deno-lint-ignore no-explicit-any
    callback) {
        return Promise.all([
            FetchpStatus.ERROR,
            request,
            undefined,
            error,
            callback(FetchpStatus.ERROR, error),
        ]);
    }
    internalLoadStep1CheckState(status, request, response, 
    // deno-lint-ignore no-explicit-any
    error, init, abortController, callback) {
        var _a, _b, _c, _d;
        // status !== FetchpStatus.CANCELED &&
        if (abortController.signal.aborted) {
            return Promise.all([
                FetchpStatus.CANCELED,
                request,
                response,
                error,
                callback(FetchpStatus.CANCELED),
                (_a = init === null || init === void 0 ? void 0 : init.cancelCallback) === null || _a === void 0 ? void 0 : _a.call(init),
                (_b = init === null || init === void 0 ? void 0 : init.statusCallback) === null || _b === void 0 ? void 0 : _b.call(init, FetchpStatus.CANCELED),
                this.hooks.callHooksWithRequest(hook_registry_js_1.FetchpHookType.StateChange, request, this.internalUrlConverter, request, FetchpStatus.CANCELED),
                this.hooks.callHooksWithRequest(hook_registry_js_1.FetchpHookType.Cancel, request, this.internalUrlConverter, request),
            ]);
        }
        if (status === FetchpStatus.ERROR) {
            return Promise.all([
                FetchpStatus.ERROR,
                request,
                response,
                error,
                callback(FetchpStatus.ERROR),
                (_c = init === null || init === void 0 ? void 0 : init.errorCallback) === null || _c === void 0 ? void 0 : _c.call(init, error),
                this.hooks.callHooksWithRequest(hook_registry_js_1.FetchpHookType.StateChange, request, this.internalUrlConverter, request, FetchpStatus.ERROR),
                this.hooks.callHooksWithRequest(hook_registry_js_1.FetchpHookType.Error, request, this.internalUrlConverter, request, error),
            ]);
        }
        return Promise.all([
            FetchpStatus.LOADING,
            request,
            response,
            error,
            callback(FetchpStatus.LOADING),
            (_d = init === null || init === void 0 ? void 0 : init.statusCallback) === null || _d === void 0 ? void 0 : _d.call(init, FetchpStatus.LOADING),
            this.hooks.callHooksWithRequest(hook_registry_js_1.FetchpHookType.StateChange, request, this.internalUrlConverter, request, FetchpStatus.LOADING),
        ]);
    }
    // deno-lint-ignore no-explicit-any
    internalLoadStep2Deserialization(status, request, response, 
    // deno-lint-ignore no-explicit-any
    error, init, 
    // deno-lint-ignore no-explicit-any
    callback) {
        var _a, _b, _c;
        if ([FetchpStatus.CANCELED, FetchpStatus.ERROR].includes(status)) {
            return Promise.all([
                status,
                undefined,
                error,
                // callback(status, error),
            ]);
        }
        const deserialized = response !== undefined
            ? this.internalDataDeserializer(response, (_a = init === null || init === void 0 ? void 0 : init.cacheRequest) !== null && _a !== void 0 ? _a : false)
            : undefined;
        return Promise.all([
            FetchpStatus.SUCCESS,
            deserialized,
            error,
            callback(FetchpStatus.SUCCESS, error),
            (_b = init === null || init === void 0 ? void 0 : init.successCallback) === null || _b === void 0 ? void 0 : _b.call(init, deserialized),
            (_c = init === null || init === void 0 ? void 0 : init.statusCallback) === null || _c === void 0 ? void 0 : _c.call(init, FetchpStatus.SUCCESS),
            this.hooks.callHooksWithRequest(hook_registry_js_1.FetchpHookType.StateChange, request, this.internalUrlConverter, request, FetchpStatus.SUCCESS),
            this.hooks.callHooksWithRequest(hook_registry_js_1.FetchpHookType.Success, request, this.internalUrlConverter, request, deserialized),
        ]);
    }
    // deno-lint-ignore no-explicit-any
    request(method, uri, init) {
        const abortController = new AbortController();
        const [awaiterResolve, awaiter] = this.internalAwaiterGenerator();
        let init_ = init !== null && init !== void 0 ? init : {};
        let url_;
        let headers;
        let status = FetchpStatus.IDLE;
        // deno-lint-ignore no-explicit-any
        let error;
        let request;
        // -- REQUEST PART -- //
        const promise = awaiter
            .then(() => {
            url_ = this.internalUrlConverter(uri, init_.uriParams);
            headers = new Headers(init_.headers);
            if (!headers.has("Content-Type")) {
                headers.set("Content-Type", "application/json");
            }
        })
            .then(() => this.internalRequestStep1InitHeaders(headers, init_, (newStatus) => (status = newStatus)))
            .then(() => this.internalRequestStep2InitRequest(method, url_, headers, init_, abortController, (newStatus, newRequest) => {
            status = newStatus;
            request = newRequest;
        }))
            .then(([, req]) => this.internalRequestStep3CheckForMocks(req, abortController, (newStatus) => (status = newStatus)))
            .then(([, req, res]) => this.internalRequestStep4CheckForCache(req, res, abortController, (newStatus) => (status = newStatus)))
            .then(([, req, res]) => this.internalRequestStep5ExecuteRequest(req, res, init_, abortController, (newStatus) => (status = newStatus)))
            .catch((err) => this.internalRequestOnError(request, err, (newStatus, newError) => {
            status = newStatus;
            error = newError;
        }));
        const response = promise.then(([, , res]) => res);
        // -- LOAD PART -- //
        const data = promise
            .then(([state, req, res, err]) => this.internalLoadStep1CheckState(state, req, res, err, init_, abortController, (newStatus) => (status = newStatus)))
            .then(([state, req, res, err]) => this.internalLoadStep2Deserialization(state, req, res, err, init_, (newStatus, newError) => {
            status = newStatus;
            error = newError;
        }))
            .then(([, data]) => data);
        const result = {
            get request() {
                return request;
            },
            get response() {
                return response;
            },
            abortController,
            get status() {
                return status;
            },
            get error() {
                return error;
            },
            get data() {
                return data;
            },
            exec: (init) => {
                init_ = Object.assign(Object.assign({}, init_), (init !== null && init !== void 0 ? init : {}));
                awaiterResolve(undefined);
                return result;
            },
        };
        if ((init === null || init === void 0 ? void 0 : init.immediate) !== false) {
            awaiterResolve(undefined);
        }
        return result;
    }
}
exports.Fetchp = Fetchp;
// singleton instance for predefined, default fetchp object
const fetchp = new Fetchp();
exports.fetchp = fetchp;
exports.default = fetchp;
