import { type FetchpURI } from "./uris.js";
import { type FetchpHookFn, FetchpHookType, HookRegistry, type HookRegistryInterface } from "./hook-registry.js";
import { MockRegistry, type MockRegistryInterface } from "./mock-registry.js";
import { CacheRegistry, type CacheRegistryInterface } from "./cache-registry.js";
declare enum FetchpStatus {
    IDLE = "idle",
    PREPARING = "preparing",
    FETCHING = "fetching",
    LOADING = "loading",
    ERROR = "error",
    CANCELED = "canceled",
    SUCCESS = "success"
}
interface FetchpRequestInit extends RequestInit {
    immediate?: boolean;
    uriParams?: Record<string, string | undefined>;
    cacheRequest?: boolean;
    statusCallback?: (status: FetchpStatus) => void;
    successCallback?: (data: any) => void;
    errorCallback?: (error: any) => void;
    cancelCallback?: () => void;
}
interface FetchpResultInterface<T = any> {
    readonly request: Request | undefined;
    readonly response: Promise<Response | undefined>;
    abortController: AbortController;
    readonly status: FetchpStatus;
    readonly error: any;
    readonly data: Promise<T | undefined>;
    exec(init?: FetchpRequestInit): FetchpResultInterface<T>;
}
interface FetchpInterface {
    baseUri: string | undefined;
    hooks: HookRegistryInterface;
    mocks: MockRegistryInterface;
    cache: CacheRegistryInterface;
    setBaseUri: (uri: string) => void;
    request: <T = any>(method: string, uri: FetchpURI, init?: FetchpRequestInit) => FetchpResultInterface<T>;
}
type InternalFetchState = [
    status: FetchpStatus,
    request: Request | undefined,
    response: Response | undefined,
    error: any,
    ...others: any[]
];
declare class Fetchp implements FetchpInterface {
    baseUri: string | undefined;
    hooks: HookRegistryInterface;
    mocks: MockRegistryInterface;
    cache: CacheRegistryInterface;
    constructor();
    setBaseUri(uri: string | undefined): void;
    internalMockChecker(request: Request): Promise<Response> | undefined;
    internalCacheChecker(request: Request): Promise<Response> | undefined;
    internalFetcher(request: Request): Promise<Response>;
    internalDataDeserializer<T>(response: Response, cacheMode: boolean): Promise<T>;
    internalUrlConverter(uri: FetchpURI, params?: Record<string, string | undefined>): URL;
    internalAwaiterGenerator<T = unknown>(): [(value: T) => void, Promise<T>];
    internalRequestStep1InitHeaders(headers: Headers, init: FetchpRequestInit | undefined, callback: (status: FetchpStatus) => void): Promise<InternalFetchState>;
    internalRequestStep2InitRequest(method: string, url: URL, headers: Headers, init: FetchpRequestInit | undefined, abortController: AbortController, callback: (status: FetchpStatus, request: Request) => void): Promise<InternalFetchState>;
    internalRequestStep3CheckForMocks(request: Request | undefined, abortController: AbortController, _callback: (status: FetchpStatus) => void): Promise<InternalFetchState>;
    internalRequestStep4CheckForCache(request: Request | undefined, mockedResponse: Response | undefined, abortController: AbortController, _callback: (status: FetchpStatus) => void): Promise<InternalFetchState>;
    internalRequestStep5ExecuteRequest(request: Request | undefined, mockedOrCachedResponse: Response | undefined, init: FetchpRequestInit | undefined, abortController: AbortController, callback: (status: FetchpStatus) => void): Promise<InternalFetchState>;
    internalRequestOnError(request: Request | undefined, error: any, callback: (status: FetchpStatus, error: any) => void): Promise<InternalFetchState>;
    internalLoadStep1CheckState(status: FetchpStatus, request: Request | undefined, response: Response | undefined, error: any, init: FetchpRequestInit | undefined, abortController: AbortController, callback: (status: FetchpStatus) => void): Promise<InternalFetchState>;
    internalLoadStep2Deserialization<T = any>(status: FetchpStatus, request: Request | undefined, response: Response | undefined, error: any, init: FetchpRequestInit | undefined, callback: (status: FetchpStatus, error: any) => void): Promise<[FetchpStatus, T | undefined, any, ...any[]]>;
    request<T = any>(method: string, uri: FetchpURI, init?: FetchpRequestInit): {
        readonly request: Request | undefined;
        readonly response: Promise<Response | undefined>;
        abortController: AbortController;
        readonly status: FetchpStatus;
        readonly error: any;
        readonly data: Promise<T | undefined>;
        exec: (init?: FetchpRequestInit) => any;
    };
}
declare const fetchp: Fetchp;
export { CacheRegistry, type CacheRegistryInterface, Fetchp, fetchp, fetchp as default, type FetchpHookFn, FetchpHookType, type FetchpInterface, type FetchpRequestInit, type FetchpResultInterface, FetchpStatus, HookRegistry, type HookRegistryInterface, MockRegistry, type MockRegistryInterface, };
