import * as dntShim from "./_dnt.shims.js";
import { type FetchpURI, type UrlCollectionInterface } from "./uris.js";
declare enum FetchpHookType {
    BuildRequestHeaders = "BuildRequestHeaders",
    NewRequest = "NewRequest",
    StateChange = "StateChange",
    Success = "Success",
    Error = "Error",
    Cancel = "Cancel"
}
type FetchpHookFn = (...params: any[]) => void;
interface FetchpHookItem {
    globals: Set<FetchpHookFn>;
    urlBased: UrlCollectionInterface<FetchpHookFn>;
}
interface HookRegistryInterface {
    items: Record<FetchpHookType, FetchpHookItem>;
    add(hookType: FetchpHookType, func: FetchpHookFn): void;
    addForUrl(hookType: FetchpHookType, methods: string | string[], requestUrlPattern: string | RegExp | dntShim.URLPattern, func: FetchpHookFn): void;
    clear(): void;
    callGlobalHooks(hookType: FetchpHookType, ...params: any[]): Promise<undefined>;
    callHooksWithRequest(hookType: FetchpHookType, request: Request | undefined, urlConverter: ((uri: FetchpURI) => URL) | undefined, ...params: any[]): Promise<undefined>;
}
declare class HookRegistry implements HookRegistryInterface {
    items: Record<FetchpHookType, FetchpHookItem>;
    constructor();
    internalEnsureHookItemExists(hookType: FetchpHookType): void;
    add(hookType: FetchpHookType, func: FetchpHookFn): void;
    addForUrl(hookType: FetchpHookType, methods: string | string[], requestUrlPattern: string | RegExp | dntShim.URLPattern, func: FetchpHookFn): void;
    clear(): void;
    callGlobalHooks(hookType: FetchpHookType, ...params: any[]): Promise<undefined>;
    callHooksWithRequest(hookType: FetchpHookType, request: Request | undefined, urlConverter: ((uri: FetchpURI) => URL) | undefined, ...params: any[]): Promise<undefined>;
}
export { type FetchpHookFn, FetchpHookType, HookRegistry, type HookRegistryInterface, };
